@@ -20,7 +20,7 @@ from torch.optim import AdamW
 from torch.optim.lr_scheduler import LambdaLR
 from torch.utils.data import DataLoader
 
-from accelerate import Accelerator
+from accelerate.accelerator import Accelerator
 from accelerate.test_utils import RegressionDataset, RegressionModel
 from accelerate.utils import DistributedType, set_seed
 
@@ -228,9 +228,38 @@ def test_gradient_accumulation_with_opt_and_scheduler(split_batches=False, dispa
         torch.manual_seed(1337 + iteration)
 
 
+def test_dataloader_break():
+    accelerator = Accelerator()
+
+    first_dset = RegressionDataset(length=80)
+    first_dataloader = DataLoader(first_dset, batch_size=16)
+    second_dset = RegressionDataset(length=96)
+    second_dataloader = DataLoader(second_dset, batch_size=16)
+    first_dataloader, second_dataloader = accelerator.prepare(first_dataloader, second_dataloader)
+    for iteration, _ in enumerate(first_dataloader):
+        # Will be True except if we are on the last batch
+        if iteration < len(first_dataloader) - 1:
+            assert id(accelerator.gradient_state.active_dataloader) == id(first_dataloader)
+            if iteration == 1:
+                for batch_num, _ in enumerate(second_dataloader):
+                    if batch_num < len(second_dataloader) - 1:
+                        assert id(accelerator.gradient_state.active_dataloader) == id(
+                            second_dataloader
+                        ), f"First dataloader: {id(first_dataloader)}\nSecond dataloader: {id(second_dataloader)}\nActive dataloader: {id(accelerator.gradient_state.active_dataloader)}\n"
+                    else:
+                        assert id(accelerator.gradient_state.active_dataloader) == id(
+                            first_dataloader
+                        ), f"First dataloader: {id(first_dataloader)}\nSecond dataloader: {id(second_dataloader)}\nActive dataloader: {id(accelerator.gradient_state.active_dataloader)}\n"
+        else:
+            assert accelerator.gradient_state.active_dataloader is None
+
+
 def main():
     accelerator = Accelerator()
     state = accelerator.state
+    if state.local_process_index == 0:
+        print("**Test `accumulate` gradient accumulation with dataloader break**")
+    test_dataloader_break()
     if state.distributed_type == DistributedType.NO:
         if state.local_process_index == 0:
             print("**Test NOOP `no_sync` context manager**")
